#include <iostream>
#include <unordered_map>
#include <random>

#include "io.h"
#include "sims.h"
#include "motions/random.h"
#include "motions/tetrahedral.h"
#include "times/delta.h"


int main (const int argc, char *argv[]) {
    Arguments args;
    try {
        args = parse_args(argc, argv);
    } catch (std::runtime_error& error) {
        std::cerr << error.what() << std::endl;
        return 1;
    }

    std::unordered_map parameter { read_parameter(args.parameter_file) };

    std::random_device rd;
    std::mt19937_64 rng(rd());

    auto motion = TetrahedralJump(rng);
    auto dist = DeltaDistribution(rng);

    if (args.spectrum) {
       run_spectrum(parameter, motion, dist);
    }
    if (args.ste) {
        run_ste(parameter, motion, dist);
    }
}
